/********************************* (C) ROTEK ***********************************
 * @module  web_params_api
 * @file    web_params_api.c
 * @version 1.0.0
 * @date    XX.XX.XXXX
 * $brief   web_params_api
 *******************************************************************************
 * @history     Version  Author         Comment
 * XX.XX.XXXX   1.0.0    Telenkov D.A.  First release.
 *******************************************************************************
 */

#include "stm32f4xx.h"  
#include "web_params_api.h"
#include "parameters.h"
#include "trap_params.h"
#include "snmp_api.h"
#include "trap_api.h"
#include "settings_api.h"
#include "common_config.h"
#include "snmp.h"

#include "FreeRTOS.h"
#include "task.h"

#ifdef PRINTF_STDLIB
#include <stdio.h>
#endif
#ifdef PRINTF_CUSTOM
#include "tinystdio.h"
#endif
#include <string.h>

typedef  void (*pFunction)(void);
pFunction Jump_To_Application;
uint32_t JumpAddress;

/**
  * @brief  Возвращяет строку с настройками на первой странице
  * @retval None
  */
void HTTP_GetParamsPage1(char *buf)
{
  char str[40];
  uint8_t len;
  memset(buf, 0, 1000);
  
  // Headers для поддержки saffari 
  strcpy(buf, "HTTP/1.0 200 OK\r\nContent-Type:text/html\r\n\r\n");
  
  // Параметры UPS
  
  GetInputVoltageStr(str, &len);
  strcat(buf, "{\"AC\":\"");
  strncat(buf, str, len);
  
  GetOutputVoltageStr(str, &len);
  strcat(buf, "\",\"DC\":\"");
  strncat(buf, str, len);
  
  GetInputFreqStr(str, &len);
  strcat(buf, "\",\"in_freq\":\"");
  strncat(buf, str, len);
  
  GetOutputFreqStr(str, &len);
  strcat(buf, "\",\"out_freq\":\"");
  strncat(buf, str, len);
  
  GetPowerStr(str, &len);
  strcat(buf, "\",\"pwr\":\"");
  strncat(buf, str, len);
  
  GetLineFailBitStr(str, &len);
  strcat(buf, "\",\"line_fail\":");
  strncat(buf, str, len);
  
  GetLowBatteryBitStr(str, &len);
  strcat(buf, ",\"low_battery\":");
  strncat(buf, str, len);
  
  GetBatCapacityStr(str, &len);
  strcat(buf, ",\"bat_cap\":\"");
  strncat(buf, str, len);
  
  GetInternalTempStr(str, &len);
  strcat(buf, "\",\"inner_temp\":\"");
  strncat(buf, str, len);
  
  GetRuntimeStr(str, &len);
  strcat(buf, "\",\"bat_time_left\":\"");
  strncat(buf, str, len);
 
  // Дополнительные параметры
  
  // Мониторинг
  // load_monitor (нагрузка, 0 - норма, 1 - fail)
  // temp_monitor (внутренняя температура, 0 - норма, 1 - fail)
  // connect_monitor (связь с UPSом, 0 - норма, 1 - fail)
  
  GetLoadMonitorStr(str, &len);
  strcat(buf, "\",\"load_monitor\":");
  strncat(buf, str, len);
  
  GetTempMonitorStr(str, &len);
  strcat(buf, ",\"temp_monitor\":");
  strncat(buf, str, len);
  
  GetConnectMonitorStr(str, &len);
  strcat(buf, ",\"connect_monitor\":");
  strncat(buf, str, len);
  
  // Признак изменения сетевых настроек 
  GetWebReinitFlag(str, &len);
  strcat(buf, ",\"netsettings_changed\":\"");
  strncat(buf, str, len);

  strncat(buf, "\"}", 2);
  
  //printf(buf);
}

/**
  * @brief  Возвращяет строку с настройками на второй странице
  * @retval None
  */
void HTTP_GetSettings(char *buf)
{
  char str[30];
  uint8_t len;
   
  /* Headers для поддержки saffari */
  strcpy(buf, "HTTP/1.0 200 OK\r\nContent-Type:text/html\r\n\r\n");
  
  /* SNMP */
  GetReadCommunity(str, &len);  
  strcat(buf, "{\"read_community\":\"");
  strncat(buf, str, len);	
  
  GetWriteCommunity(str, &len);  
  strcat(buf, "\",\"write_community\":\"");
  strncat(buf, str, len);
  
  GetManagerIp(str, &len);  
  strcat(buf, "\",\"managerIP\":\"");
  strncat(buf, str, len);
  
  GetManagerIp2(str, &len);  
  strcat(buf, "\",\"managerIP2\":\"");
  strncat(buf, str, len);
  
  GetManagerIp3(str, &len);  
  strcat(buf, "\",\"managerIP3\":\"");
  strncat(buf, str, len);
	
  /* WEB */
  GetIpStr(str, &len);  
  strcat(buf, "\",\"ipaddr\":\"");
  strncat(buf, str, len);
  
  GetGatewayStr(str, &len);  
  strcat(buf, "\",\"gw\":\"");
  strncat(buf, str, len);
  
  GetMaskStr(str, &len);  
  strcat(buf, "\",\"mask\":\"");
  strncat(buf, str, len);
  
  GetDhcpStateStr(str, &len);  
  strcat(buf, "\",\"dhcp\":");
  strncat(buf, str, len);
 
  /* Признак изменения сетевых настроек */ 
  GetWebReinitFlag(str, &len);
  strcat(buf, ",\"netsettings_changed\":\"");
  strncat(buf, str, len);
  
  strncat(buf, "\"}", 2);
  
  //printf(buf);
}

/**
  * @brief  Возвращяет строку с информацией об устройстве
  * @retval None
  */
// TODO Согласовать максимальную длину строк
void HTTP_GetInfo(char *buf)
{
  char str[40];
  uint8_t len;
  
  /* Headers для поддержки saffari */
  strcpy(buf, "HTTP/1.0 200 OK\r\nContent-Type:text/html\r\n\r\n");
  
  GetWorkTimeStr(str, &len);  
  strcat(buf, "{\"uptime\":\"");
  strncat(buf, str, len);	
  
  GetModelStr(str, &len);  
  strcat(buf, "\",\"model\":\"");
  strncat(buf, str, len);
  
  GetProductionDataStr(str, &len);  
  strcat(buf, "\",\"prodate\":\"");
  strncat(buf, str, len);
  
  GetVersionStr(str, &len);  
  strcat(buf, "\",\"fwversion\":\"");
  strncat(buf, str, len);
  
  GetMacStr(str, &len);  
  strcat(buf, "\",\"macaddr\":\"");
  strncat(buf, str, len);
  
  GetSerialNumberStr(str, &len);  
  strcat(buf, "\",\"serno\":\"");
  strncat(buf, str, len);
  
  GetOwnerStr(str, &len);  
  strcat(buf, "\",\"owner\":\"");
  strncat(buf, str, len);
  
  GetLocationStr(str, &len);  
  strcat(buf, "\",\"sysLocation\":\"");
  strncat(buf, str, len);
  
  GetCommentsStr(str, &len);  
  strcat(buf, "\",\"comment\":\"");
  strncat(buf, str, len);
 
  /* Признак изменения сетевых настроек */ 
  GetWebReinitFlag(str, &len);
  strcat(buf, "\",\"netsettings_changed\":\"");
  strncat(buf, str, len);
  
  strncat(buf, "\"}", 2);
  
}


/**
  * @brief  Возвращает uptime, freq, dutycicle
  */
void HTTP_GetProgonParams(char *buf)
{
  char str[20];
  u32_t tick;  
  
  strcpy(buf, "HTTP/1.0 200 OK\r\nContent-Type:text/html\r\n\r\n");
    
  strcat(buf, "{\"uptime\":\"");
  snmp_get_sysuptime(&tick);
  tick = tick/100;
  memset(str, 0, 20);
  sprintf(str, "%d", tick);
  strncat(buf, str, strlen(str));

  strncat(buf, "\"}", 2);
}





/**
  * @brief  Сброс настроек (всё кроме сетевых параметров)
  */
void HTTP_ResetSettings(void)
{
  //taskENTER_CRITICAL();
    
  SETTINGS_SetPartDefault();
  SETTINGS_Save();
	  
  //taskEXIT_CRITICAL();
}

/**
  * @brief  Сброс настроек
  */
void HTTP_SaveSettings(void)
{
//  taskENTER_CRITICAL();
  
  SETTINGS_Save();
  
//  taskEXIT_CRITICAL();
  
  //SETTINGS_ApplyAlgorithmParams();
}

/**
  * @brief  Перезагрузка контроллера
  */
void HTTP_Reboot(void)
{
  vTaskDelay(1010);
  NVIC_SystemReset();
}

/**
  * @brief  Запускает задачу ресета контроллера
  * @param fBootMode:
  *        true - перед перезагрузкой будет установлен флаг входа в режим 
  *        bootloader и перезаписаны настройки
  *
  *        false - обычная перезагрузка 
  */
void HTTP_StartResetTask(bool fBootMode)
{
  static bool temp;
 
  temp = fBootMode;
  xTaskCreate(vTaskReboot, "RebootTask", configMINIMAL_STACK_SIZE, 
			  (void*)&temp, tskIDLE_PRIORITY, NULL);
}

/**
  * @brief  
  */
void vTaskReboot(void * pvParameters)
{
  bool mode = false;  // тип перезагрузки (нужно ли устанавливать флаг bootloader)
  
  for (;;)
  {
	mode = *(bool*)pvParameters;
	
	if (mode)
	{
	  SetLoadMode();
	  HTTP_SaveSettings();
	  SNMP_SendUserTrap(FW_VERSION_UPDATE);
      vTaskDelay(1000);
      /* Блокируем управление ключем на тау секунд*/
      //IO_KeyBlockOn();
      vTaskDelay(1010);
	  NVIC_SystemReset();
	}  
	else
	{
	  vTaskDelay(1000);
      /* Блокируем управление ключем на тау секунд*/
      //IO_KeyBlockOn();
      vTaskDelay(1010);
	  NVIC_SystemReset();
	}
  }	
}

/********************************* (C) ROTEK **********************************/
