/********************************* (C) РОТЕК ***********************************
 * @module  parameters
 * @file    parameters.h
 * @version 1.0.0
 * @date    XX.XX.XXXX
 * $brief   parameters
 *******************************************************************************
 * @history     Version  Author         Comment
 * XX.XX.XXXX   1.0.0    Telenkov D.A.  First release.
 *******************************************************************************
 */

/* Define to prevent recursive  ----------------------------------------------*/
#ifndef PARAMETERS_H
#define PARAMETERS_H

#include <stdbool.h>

// ************************************************************************** //
//                        Параметры UPS

void GetLineFailBitStr(char *str, uint8_t *len);
void GetLowBatteryBitStr(char *str, uint8_t *len);
void GetInputFreqStr(char *str, uint8_t *len);
void GetOutputFreqStr(char *str, uint8_t *len);
void GetInputVoltageStr(char *str, uint8_t *len);
void GetOutputVoltageStr(char *str, uint8_t *len);
void GetPowerStr(char *str, uint8_t *len);
void GetBatCapacityStr(char *str, uint8_t *len);
void GetRuntimeStr(char *str, uint8_t *len);
void GetInternalTempStr(char *str, uint8_t *len);
void GetLoadMonitorStr(char *str, uint8_t *len);
void GetTempMonitorStr(char *str, uint8_t *len);
void GetConnectMonitorStr(char *str, uint8_t *len);



/**
  * @brief  Контроль входного напряжения ИИП
  *         0 - напряжение в норме;
  *         1 - напряжение ниже нормы
  *         2 - напряжение выше нормы
  */
void GetAcStr(char *str, uint8_t *len);

/**
  * @brief  Контроль входного напряжения ИИП
  */
void GetAcInt(char *str, uint8_t *len);

/**
  * @brief  Напряжение на выходе ИИП
  */
void GetPpsVoltStr(char *str, uint8_t *len);

/**
  * @brief  Выходной ток ИИП
  */
void GetPpsCurrStr(char *str, uint8_t *len);

/**
  * @brief  Управление ИИП (отключен/подключен)
  */
void GetPpsControlStr(char *str, uint8_t *len);

/**
  * @brief  Управление ИИП (отключен/подключен)
  */
void GetPpsControlInt(uint32_t *value);

/**
  * @brief  Качество АКБ
  */
void GetBatQualityStr(char *str, uint8_t *len);

/**
  * @brief  Оставшееся время работы АКБ
  */
void GetBatTimeLeftStr(char *str, uint8_t *len);

/**
  * @brief  Температура на АКБ
  */
void GetBatTempStr(char *str, uint8_t *len);

/**
  * @brief  Температура на АКБ
  */
void GetBatTempStrEn(char *str, uint8_t *len);

/**
  * @brief  Напряжение на АКБ
  */
void GetBatVoltStr(char *str, uint8_t *len);

/**
  * @brief  Напряжение на 1й батареи
  */
void GetBatVolt1Str(char *str, uint8_t *len);

/**
  * @brief  Напряжение на 2й батареи
  */
void GetBatVolt2Str(char *str, uint8_t *len);

/**
  * @brief  Напряжение на 3й батареи
  */
void GetBatVolt3Str(char *str, uint8_t *len);

/**
  * @brief  Напряжение на 4й батареи
  */
void GetBatVolt4Str(char *str, uint8_t *len);

/**
  * @brief  Ток заряда
  */
void GetBatCurrStr(char *str, uint8_t *len);

/**
  * @brief  Симметрия АКБ
  */
void GetBatSymStr(char *str, uint8_t *len);

/**
  * @brief  Симметрия АКБ
  */
void GetBatSymInt(char *str, uint8_t *len);

/**
  * @brief  Отключение питания от АКБ
  */
void GetBatToMcuKeyStr(char *str, uint8_t *len);

/**
  * @brief  Отключение питания от АКБ
  */
void GetBatToMcuKeyInt(uint32_t *value);

/**
  * @brief  Отключение АКБ от зарядки
  */
void GetBatToSupKeyStr(char *str, uint8_t *len);

/**
  * @brief  Отключение АКБ от зарядки
  */
void GetBatToSupKeyInt(uint32_t *value);

/**
  * @brief  Напряжение на нагрузке
  */
void GetLoadVoltStr(char *str, uint8_t *len);

/**
  * @brief  Ток в нагрузку
  */
void GetLoadCurrStr(char *str, uint8_t *len);

/**
  * @brief  Управление нагрузкой
  */
void GetLoadKeyStr(char *str, uint8_t *len);

/**
  * @brief  Управление нагрузкой
  */
void GetLoadKeyInt(uint32_t *value);

/**
  * @brief  Температура за бортом
  */
void GetExtTempStr(char *str, uint8_t *len);

/**
  * @brief  Температура в шкафу
  */
void GetExtTempStrEn(char *str, uint8_t *len);

/**
  * @brief  Режим работы
  */
void GetModeStr(char *str, uint8_t *len);

/**
  * @brief  Режим работы
  */
void GetModeEn(char *str, uint8_t *len);

/**
  * @brief  Индикация аварийных ситуаций
  * TODO добавить реальный параметр
  */
void GetAlarmStr(char *str, uint8_t *len);

/**
  * @brief  Состояние двери
  */
void GetDoorStr(char *str, uint8_t *len);

/**
  * @brief  Состояние двери
  */
void GetDoorInt(char *str, uint8_t *len);

/**
  * @brief  Текущее время ч/м/с
  */
void GetCurrentTimeStr(char *str, uint8_t *len);

// ************************************************************************** //
//                             Информация

/**
  * @brief  Время работы устройства
  */
void GetWorkTimeStr(char *str, uint8_t *len);

/**
  * @brief  Модель устройства
  */
void GetModelStr(char *str, uint8_t *len);

/**
  * @brief  Дата производства
  */
void GetProductionDataStr(char *str, uint8_t *len);

/**
  * @brief  Версия ПО
  */
void GetVersionStr(char *str, uint8_t *len);

/**
  * @brief  MAC адрес устройства
  */
void GetMacStr(char *str, uint8_t *len);

/**
  * @brief  Серийный номер устройства
  */
void GetSerialNumberStr(char *str, uint8_t *len);

/**
  * @brief  Владелец устройства
  */
void GetOwnerStr(char *str, uint8_t *len);

/**
  * @brief  Местоположение устройства
  */
void GetLocationStr(char *str, uint8_t *len);

/**
  * @brief  Комментарии
  */
void GetCommentsStr(char *str, uint8_t *len);


// ************************************************************************** //
//                        Параметры температуры

/**
  * @brief  Нижняя граница рабочей температуры (в шкафу)
  */
void GetTempMinStr(char *str, uint8_t *len);

/**
  * @brief  Верхняя граница рабочей температуры (в шкафу)
  */
void GetTempMaxStr(char *str, uint8_t *len);

/**
  * @brief  Верхняя граница рабочей температуры (в шкафу)
  */
void GetTempHistMinStr(char *str, uint8_t *len);

/**
  * @brief  Верхняя граница рабочей температуры (в шкафу)
  */
void GetTempHistMaxStr(char *str, uint8_t *len);

// ************************************************************************** //
//                   Параметры температуры батарей

/**
  * @brief  Состояние контроля температуры батарей (вкл/выкл)
  */
void GetBatTempStateStr(char *str, uint8_t *len);

/**
  * @brief  Нижняя граница рабочей температуры (на батареях)
  */
void GetBatTempMinStr(char *str, uint8_t *len);

/**
  * @brief  Верхняя граница рабочей температуры (на батареях)
  */
void GetBatTempMaxStr(char *str, uint8_t *len);

/**
  * @brief  Гистерезис для нижней границы рабочей температуры (на батареях)
  */
void GetBatTempHistMinStr(char *str, uint8_t *len);

/**
  * @brief  Гистерезис для врхней границы рабочей температуры (на батареях)
  */
void GetBatTempHistMaxStr(char *str, uint8_t *len);

/**
  * @brief  Напряжение отклчения АКБ
  */
void GetBatVoltageMinStr(char *str, uint8_t *len);

/**
  * @brief  Напряжение отклчения АКБ гистерезис
  */
void GetBatVoltageMinHistStr(char *str, uint8_t *len);

/**
  * @brief  Емкость
  */
void GetBatCapasityStr(char *str, uint8_t *len);

/**
  * @brief  Напряжение заряда АКБ
  */
void GetBatChargeVoltStr(char *str, uint8_t *len);

/**
  * @brief  Вкл/выкл температурную компенсацию АКБ
  */
void GetBatTempCompensStr(char *str, uint8_t *len);

/**
  * @brief  Коэффициент температурной коменсации в буферном режиме
  */
void GetBatCoefBufStr(char *str, uint8_t *len);

/**
  * @brief  Коэффициент температурной коменсации в циклическом режиме
  */
void GetBatCoefCycleStr(char *str, uint8_t *len);

/**
  * @brief  Вкл/выкл контроль симметрии АКБ
  */
void GetBatSymmetryStr(char *str, uint8_t *len);

/**
  * @brief  Напряжение отключения нагрузки
  */
void GetLoadVoltMinStr(char *str, uint8_t *len);

/**
  * @brief  Напряжение отключения нагрузки гистерезис
  */
void GetLoadVoltMinHistStr(char *str, uint8_t *len);

/**
  * @brief  Максимальный ток
  */
void GetPpsCurrentMaxStr(char *str, uint8_t *len);

/**
  * @brief  Максимальный ток гистерезис
  */
void GetPpsCurrentMaxHistStr(char *str, uint8_t *len);

// ************************************************************************** //
//                             Параметры WEB

/**
  * @brief  Текущий IP адрес
  */
void GetIpStr(char *str, uint8_t *len);

/**
  * @brief  Текущий шлюз
  */
void GetGatewayStr(char *str, uint8_t *len);

/**
  * @brief  Текущая маска подсети
  */
void GetMaskStr(char *str, uint8_t *len);

/**
  * @brief  Настройка DHCP
  */
void GetDhcpStateStr(char *str, uint8_t *len);

// ************************************************************************** //
//                             Параметры SNMP
/**
  * @brief  Текущее TRAP community
  */
void GetTrapCommunity(char *str, uint8_t *len);

/**
  * @brief  Текущее community для чтения
  */
void GetReadCommunity(char *str, uint8_t *len);

/**
  * @brief  Текущее community для записи
  */
void GetWriteCommunity(char *str, uint8_t *len);

/**
  * @brief  IP SNMP менеджера
  */
void GetManagerIp(char *str, uint8_t *len);
void GetManagerIp2(char *str, uint8_t *len);
void GetManagerIp3(char *str, uint8_t *len);

// ************************************************************************** //
//                             Параметры SNTP
/**
  * @brief  Состояние SNTP
  */
void GetSntpStateStr(char *str, uint8_t *len);

/**
  * @brief  IP адрес SNTP сервера
  */
void GetSntpServerIpStr(char *str, uint8_t *len);

/**
  * @brief  Часовой пояс контроллера
  */
void GetSntpTimeZoneStr(char *str, uint8_t *len);

/**
  * @brief  Дата последней синхронизации SNTP
  */
void GetSntpLastDataStr(char *str, uint8_t *len);


// ************************************************************************** //
//                          Сброс настроек и перезагрузка

/**
  * @brief  Произошел сброс настроек
  */
void GetRestore(char *str, uint8_t *len);

/**
  * @brief  Произошла перезагрузка
  */
void GetReboot(char *str, uint8_t *len);

// ************************************************************************** //
//                                Флаги

/**
  * @brief  Установить признак изменения сетевых параметров
  */
void GetWebReinitFlag(char *str, uint8_t *len);

/**
  * @brief  Возвращает флаг необходимости профилактики сульфатации
  */
void GetProfilactionFlag(char *str, uint8_t *len);

/**
  * @brief  Возвращает флаг необходимости профилактики сульфатации
  */
void GetProfilactionFlagInt(uint32_t *value);

/**
  * @brief  Возвращает флаг необходимости профилактики сульфатации
  */
void GetProfilactionFlagSnmp(char* value, uint8_t *len);

/**
  * @brief  Возвращает частоту
  */
void GetFreqStr(char *str, uint8_t *len);

/**
  * @brief  Возвращает dutycycle
  */
void GetDutyStr(char *str, uint8_t *len);

// ************************************************************************** //
//                             Установка параметрв
// ************************************************************************** //

/**
  * @brief  Установить признак обновления ПО. 
  */
void SetLoadMode(void);

/**
  * @brief  Установить признак изменения сетевых параметров
  */
void SetWebReinitFlag(bool value);

/**
  * @brief  Установить флаг подтверждения сетевых настроек пользователем
  */
void SetConfirmWebParamsFlag(void);

// ************************************************************************** //
//                 Параметры контроля температуры в шкафу
/**
  * @brief  Нижняя граница рабочей температуры (в шкафу)
  */
void SetTempMinStr(char *str);

/**
  * @brief  Верхняя граница рабочей температуры (в шкафу)
  */
void SetTempMaxStr(char *str);

/**
  * @brief  Верхняя граница рабочей температуры гистерезис (в шкафу)
  */
void SetTempHistMinStr(char *str);

/**
  * @brief  Верхняя граница рабочей температуры гистерезис (в шкафу)
  */
void SetTempHistMaxStr(char *str);

// ************************************************************************** //
//                Параметры контроля температуры на батареях

/**
  * @brief  Состояние контроля температуры батарей (вкл/выкл)
  */
void SetBatTempStateStr(char *str);

/**
  * @brief  Нижняя граница рабочей температуры (на батареях)
  */
void SetBatTempMinStr(char *str);

/**
  * @brief  Верхняя граница рабочей температуры (на батареях)
  */
void SetBatTempMaxStr(char *str);

/**
  * @brief  Гистерезис для нижней границы рабочей температуры (на батареях)
  */
void SetBatTempHistMinStr(char *str);

/**
  * @brief  Гистерезис для врхней границы рабочей температуры (на батареях)
  */
void SetBatTempHistMaxStr(char *str);

// ************************************************************************** //
//                             Параметры АКБ

/**
  * @brief  Напряжение отклчения АКБ
  */
void SetBatVoltageMinStr(char *str);

/**
  * @brief  Напряжение отклчения АКБ гистерезис
  */
void SetBatVoltageMinHistStr(char *str);

/**
  * @brief  Емкость АКБ
  */
void SetBatCapasityStr(char *str);

/**
  * @brief  Напряжение заряда АКБ
  */
void SetBatChargeVoltStr(char *str);

/**
  * @brief  Вкл/выкл температурную компенсацию АКБ
  */
void SetBatTempCompensStr(char *str);

/**
  * @brief  Коэффициент температурной коменсации в буферном режиме
  */
void SetBatCoefBufStr(char *str);

/**
  * @brief  Коэффициент температурной коменсации в циклическом режиме
  */
void SetBatCoefCycleStr(char *str);

/**
  * @brief  Вкл/выкл контроль симметрии АКБ
  */
void SetBatSymmetryStr(char *str);

// ************************************************************************** //
//                             Сетевые параметры

/**
  * @brief  IP адрес
  */
void SetIPStr(char *str);

/**
  * @brief  Шлюз
  */
void SetGatewayStr(char *str);


/**
  * @brief  Текущая маска подсети
  */
void SetMaskStr(char *str);

/**
  * @brief  Настройка DHCP
  */
void SetDhcpStateStr(char *str);

// ************************************************************************** //
//                             Параметры SNMP

/**
  * @brief  TRAP community
  */
void SetTrapCommunity(char *str);

/**
  * @brief  Community для чтения
  */
void SetReadCommunity(char *str);

/**
  * @brief  Community для записи
  */
void SetWriteCommunity(char *str);

/**
  * @brief  IP SNMP менеджера
  */
void SetManagerIp(char *str);
void SetManagerIp2(char *str);
void SetManagerIp3(char *str);

/**
  * @brief  Установить владельца
  */
void SetOwner(char *str);

/**
  * @brief  Установить местоположение
  */
void SetLocation(char *str);

/**
  * @brief  Установить комментарий
  */
void SetComment(char *str);


// ************************************************************************** //
//                             Параметры нагрузки

/**
  * @brief  Напряжение отключения нагрузки
  */
void SetLoadVoltMinStr(char *str);

/**
  * @brief  Напряжение отключения нагрузки гистерезис
  */
void SetLoadVoltMinHistStr(char *str);

// ************************************************************************** //
//                             Параметры ИИП
/**
  * @brief  Максимальный ток
  */
void SetPpsCurrentMaxStr(char *str);

/**
  * @brief  Максимальный ток гистерезис
  */
void SetPpsCurrentMaxHistStr(char *str);

// ************************************************************************** //
//                             Параметры SNTP

/**
  * @brief  Состояние SNTP
  */
void SetSntpStateStr(char *str);

/**
  * @brief  IP адрес SNTP сервера
  */
void SetSntpServerIpStr(char *str);

/**
  * @brief  Часовой пояс контроллера
  */
void SetSntpTimeZoneStr(char *str);


#endif /* #ifndef PARAMETERS_H */

/********************************* (C) РОТЕК **********************************/